"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NomnomlTemplate = void 0;
const typescript_1 = require("typescript");
class NomnomlTemplate {
    constructor(settings) {
        this.settings = settings;
        this.composition = "+->";
    }
    implements(interf, implementation) {
        return `${this.plainClassOrInterface(interf)}<:--${this.plainClassOrInterface(implementation)}`;
    }
    extends(base, derived) {
        return `${this.plainClassOrInterface(base)}<:-${this.plainClassOrInterface(derived)}`;
    }
    plainClassOrInterface(name) { return `[${name}]`; }
    class(name, props, methods) {
        return `[${name}|${props.map(p => this.propertyTemplate(p)).join(";")}|${methods.map(m => this.methodTemplate(m)).join(";")}]`;
    }
    interface(name, props, methods) {
        return `[<interface>${name}|${props.map(p => this.propertyTemplate(p)).join(";")}|${methods.map(m => this.methodTemplate(m)).join(";")}]`;
    }
    type(name, props, methods) {
        return `[<type>${name}|${props.map(p => this.propertyTemplate(p)).join(";")}|${methods.map(m => this.methodTemplate(m)).join(";")}]`;
    }
    enum(name, enumItems) {
        return `[<enumeration>${name}|${enumItems.join(";")}]`;
    }
    memberAssociation(association) {
        return `${this.plainClassOrInterface(association.a.name)} ${association.a.multiplicity ?? ''} - ${association.b.multiplicity ?? ''} ${this.plainClassOrInterface(association.b.name)}`;
    }
    methodTemplate(method) {
        method = escapeMethodDetails(method);
        let retVal = method.name + "()";
        if (method.returnType && this.settings.propertyTypes) {
            retVal += ": " + method.returnType;
        }
        retVal = this.modifierTemplate(method.modifierFlags) + retVal;
        return retVal;
    }
    propertyTemplate(property) {
        property = escapePropertyDetails(property);
        let retVal = property.name;
        if (property.type && this.settings.propertyTypes) {
            if (property?.optional) {
                retVal += "?";
            }
            retVal += ": " + property.type;
        }
        retVal = this.modifierTemplate(property.modifierFlags) + retVal;
        return retVal;
    }
    modifierTemplate(modifierFlags) {
        if (!this.settings.modifiers) {
            return "";
        }
        let retVal = "";
        // UML2: static member should be underlined --> Not supported by nomnoml 
        if (modifierFlags & typescript_1.ModifierFlags.Static) {
            retVal = "static ";
        }
        if (modifierFlags & typescript_1.ModifierFlags.Abstract) {
            retVal = "abstract ";
        }
        if (modifierFlags & typescript_1.ModifierFlags.Private) {
            retVal = "-" + retVal;
        }
        else if (modifierFlags & typescript_1.ModifierFlags.Protected) {
            retVal = "\\#" + retVal;
        }
        else {
            retVal = "+" + retVal;
        }
        return retVal;
    }
}
exports.NomnomlTemplate = NomnomlTemplate;
;
// utility functions
function escapeNomnoml(str) {
    return str.replace(/[|\][\#]/g, '\\$&');
}
function escapeMethodDetails(details) {
    if (details.returnType) {
        details.returnType = escapeNomnoml(details.returnType);
    }
    return details;
}
function escapePropertyDetails(details) {
    details.name = escapeNomnoml(details.name);
    if (details.type) {
        details.type = escapeNomnoml(details.type);
    }
    return details;
}
//# sourceMappingURL=nomnoml-template.js.map